@@ -1,4 +1,19 @@
 #!/usr/bin/perl
+#
+# This file is part of GNU Stow.
+#
+# GNU Stow is free software: you can redistribute it and/or modify it
+# under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# GNU Stow is distributed in the hope that it will be useful, but
+# WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
+# General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program. If not, see https://www.gnu.org/licenses/.
 
 #
 # Utilities shared by test scripts
@@ -9,32 +24,59 @@ package testutil;
 use strict;
 use warnings;
 
-use File::Path qw(remove_tree);
+use Carp qw(croak);
+use File::Basename;
+use File::Path qw(make_path remove_tree);
+use File::Spec;
+use IO::Scalar;
+use Test::More;
 
 use Stow;
 use Stow::Util qw(parent canon_path);
 
 use base qw(Exporter);
 our @EXPORT = qw(
-    $OUT_DIR
+    $ABS_TEST_DIR
+    $TEST_DIR
+    $stderr
     init_test_dirs
     cd
     new_Stow new_compat_Stow
-    make_dir make_link make_file
-    remove_dir remove_link
+    make_path make_link make_invalid_link make_file
+    remove_dir remove_file remove_link
     cat_file
+    is_link is_dir_not_symlink is_nonexistent_path
+    capture_stderr uncapture_stderr
 );
 
-our $OUT_DIR = 'tmp-testing-trees';
+our $TEST_DIR = 'tmp-testing-trees';
+our $ABS_TEST_DIR = File::Spec->rel2abs('tmp-testing-trees');
+
+our $stderr;
+my $tied_err;
+
+sub capture_stderr {
+    undef $stderr;
+    $tied_err = tie *STDERR, 'IO::Scalar', \$stderr;
+}
+
+sub uncapture_stderr {
+    undef $tied_err;
+    untie *STDERR;
+}
 
 sub init_test_dirs {
-    for my $dir ("$OUT_DIR/target", "$OUT_DIR/stow") {
-        -d $dir and remove_tree($dir);
-        make_dir($dir);
+    # Create a run_from/ subdirectory for tests which want to run
+    # from a separate directory outside the Stow directory or
+    # target directory.
+    for my $dir ("target", "stow", "run_from") {
+        my $path = "$TEST_DIR/$dir";
+        -d $path and remove_tree($path);
+        make_path($path);
     }
 
     # Don't let user's ~/.stow-global-ignore affect test results
-    $ENV{HOME} = '/tmp/fake/home';
+    $ENV{HOME} = $ABS_TEST_DIR;
 }
 
 sub new_Stow {
@@ -56,51 +98,50 @@ sub new_compat_Stow {
 # Purpose   : safely create a link
 # Parameters: $target => path to the link
 #           : $source => where the new link should point
+#           : $invalid => true iff $source refers to non-existent file
 # Returns   : n/a
 # Throws    : fatal error if the link can not be safely created
 # Comments  : checks for existing nodes
 #============================================================================
 sub make_link {
-    my ($target, $source) = @_;
+    my ($target, $source, $invalid) = @_;
 
     if (-l $target) {
         my $old_source = readlink join('/', parent($target), $source) 
-            or die "could not read link $target/$source";
+            or die "$target is already a link but could not read link $target/$source";
         if ($old_source ne $source) {
             die "$target already exists but points elsewhere\n";
         }
     }
-    elsif (-e $target) {
-        die "$target already exists and is not a link\n";
+    die "$target already exists and is not a link\n" if -e $target;
+    my $abs_target = File::Spec->rel2abs($target);
+    my $target_container = dirname($abs_target);
+    my $abs_source = File::Spec->rel2abs($source, $target_container);
+    #warn "t $target c $target_container as $abs_source";
+    if (-e $abs_source) {
+        croak "Won't make invalid link pointing to existing $abs_target"
+            if $invalid;
     }
     else {
-        symlink $source, $target
-            or die "could not create link $target => $source ($!)\n";
+        croak "Won't make link pointing to non-existent $abs_target"
+            unless $invalid;
     }
-    return;
+    symlink $source, $target
+        or die "could not create link $target => $source ($!)\n";
 }
 
 #===== SUBROUTINE ===========================================================
-# Name      : make_dir()
-# Purpose   : create a directory and any requisite parents
-# Parameters: $dir => path to the new directory
+# Name      : make_invalid_link()
+# Purpose   : safely create an invalid link
+# Parameters: $target => path to the link
+#           : $source => the non-existent source where the new link should point
 # Returns   : n/a
-# Throws    : fatal error if the directory or any of its parents cannot be
-#           : created
-# Comments  : none
+# Throws    : fatal error if the link can not be safely created
+# Comments  : checks for existing nodes
 #============================================================================
-sub make_dir {
-    my ($dir) = @_;
-
-    my @parents = ();
-    for my $part (split '/', $dir) {
-        my $path = join '/', @parents, $part;
-        if (not -d $path and not mkdir $path) {
-            die "could not create directory: $path ($!)\n";
-        }
-        push @parents, $part;
-    }
-    return;
+sub make_invalid_link {
+    my ($target, $source, $allow_invalid) = @_;
+    make_link($target, $source, 1);
 }
 
 #===== SUBROUTINE ===========================================================
@@ -185,7 +226,7 @@ sub remove_dir {
         next NODE if $node eq '..';
 
         my $path = "$dir/$node";
-        if (-l $path or -z $path or $node eq $Stow::LOCAL_IGNORE_FILE) {
+        if (-l $path or (-f $path and -z $path) or $node eq $Stow::LOCAL_IGNORE_FILE) {
             unlink $path or die "cannot unlink $path ($!)\n";
         }
         elsif (-d "$path") {
@@ -229,6 +270,41 @@ sub cat_file {
     return $contents;
 }
 
+#===== SUBROUTINE ===========================================================
+# Name      : is_link()
+# Purpose   : assert path is a symlink
+# Parameters: $path => path to check
+#           : $dest => target symlink should point to
+#============================================================================
+sub is_link {
+    my ($path, $dest) = @_;
+    ok(-l $path => "$path should be symlink");
+    is(readlink $path, $dest => "$path symlinks to $dest");
+}
+
+#===== SUBROUTINE ===========================================================
+# Name      : is_dir_not_symlink()
+# Purpose   : assert path is a directory not a symlink
+# Parameters: $path => path to check
+#============================================================================
+sub is_dir_not_symlink {
+    my ($path) = @_;
+    ok(! -l $path => "$path should not be symlink");
+    ok(-d _       => "$path should be a directory");
+}
+
+#===== SUBROUTINE ===========================================================
+# Name      : is_nonexistent_path()
+# Purpose   : assert path does not exist
+# Parameters: $path => path to check
+#============================================================================
+sub is_nonexistent_path {
+    my ($path) = @_;
+    ok(! -l $path => "$path should not be symlink");
+    ok(! -e _     => "$path should not exist");
+}
+
+
 1;
 
 # Local variables:
